import asyncio
import webbrowser

import aiohttp
import click
import dask.config
import rich

from ..exceptions import AuthenticationError
from ..utils import COILED_SERVER, handle_credentials
from .utils import CONTEXT_SETTINGS


async def make_unattached_token(server, label=None) -> dict:
    url = server + "/api/v1/api-tokens-no-user/"
    async with aiohttp.ClientSession() as session:
        async with session.post(url, data={"label": label}) as resp:
            return await resp.json()


def get_local_user() -> str:
    try:
        import getpass
        import socket

        local_user = f"{getpass.getuser()}@{socket.gethostname()}"
    except Exception:
        local_user = ""
    return local_user


async def new_token_flow(server, browser):
    server = server or dask.config.get("coiled.server", COILED_SERVER)

    token_data = await make_unattached_token(server, label=get_local_user())

    token_identifier = token_data["identifier"]
    token_secret = token_data["token"]

    url = f"{server}/activate-token?identifier={token_identifier}"

    if browser:
        webbrowser.open(url, new=2)

    rich.print(
        f"""[blue]Go to {url} to authorize this computer to use your Coiled account.
Your API token identifier is {token_identifier}."""
    )

    retries = 60
    while retries > 0:
        try:
            await handle_credentials(
                server=server, token=token_secret, save=True, retry=False, print_invalid_token_messages=False
            )
        except AuthenticationError:
            # still waiting for user to do their their
            retries -= 1
            await asyncio.sleep(1)
        else:
            break


@click.command(context_settings=CONTEXT_SETTINGS)
@click.option("--browser/--no-browser", default=True, help="Open browser with page where you grant access")
@click.option("-s", "--server", help="Coiled server to use", hidden=True)
def authenticate(browser, server):
    """Conveniently configure your Coiled account credentials"""
    asyncio.run(new_token_flow(server, browser))
